@@ -1,29 +1,31 @@
-use async_std::future::timeout;
-use async_std::sync::Arc;
-use async_std::task::{Builder, JoinHandle};
+use std::sync::Arc;
+
 use futures::future::join_all;
 use handlebars::TemplateRenderError;
 use log::{error, info, trace, warn};
 
-use chord_core::action::Action;
 use chord_core::case::{CaseAssess, CaseState};
 use chord_core::collection::TailDropVec;
 use chord_core::flow::Flow;
-use chord_core::input::CaseStore;
-use chord_core::output::Report;
+use chord_core::future::task::{spawn, JoinError, JoinHandle};
+use chord_core::future::time::timeout;
+use chord_core::input::{StageLoader, TaskLoader};
 use chord_core::output::Utc;
+use chord_core::output::{StageReporter, TaskReporter};
 use chord_core::step::{StepAssess, StepState};
-use chord_core::task::{TaskAssess, TaskId, TaskState};
+use chord_core::task::{StageAssess, StageState, TaskAssess, TaskId, TaskState};
 use chord_core::value::{json, Map, Value};
 use res::TaskAssessStruct;
 
 use crate::flow::case;
-use crate::flow::case::arg::CaseArgStruct;
-use crate::flow::render_assign_object;
-use crate::flow::step::arg::CreateArgStruct;
+use crate::flow::case::arg::{CaseArgStruct, CaseIdStruct};
+use crate::flow::step::arg::ArgStruct;
+use crate::flow::step::StepRunner;
 use crate::flow::task::arg::TaskIdSimple;
+use crate::flow::task::res::StageAssessStruct;
 use crate::flow::task::Error::*;
-use crate::model::app::{FlowApp, RenderContext};
+use crate::flow::{assign_by_render, step};
+use crate::model::app::{App, RenderContext};
 use crate::CTX_ID;
 
 pub mod arg;
@@ -34,52 +36,52 @@ enum Error {
     #[error("`{0}` render:\n{1}")]
     Render(String, TemplateRenderError),
 
-    #[error("step `{0}` create:\n{1}")]
-    StepCreate(String, Box<dyn std::error::Error + Sync + Send>),
-
     #[error("pre:")]
     PreErr,
 
     #[error("pre step `{0}`")]
     PreFail(String),
 
-    #[error("report error:\n{0}")]
-    Report(Box<dyn std::error::Error + Sync + Send>),
+    #[error("{0} `{1}` reporter error:\n{2}")]
+    Reporter(String, String, Box<dyn std::error::Error + Sync + Send>),
 
-    #[error("stage `{0}` load error:\n{1}")]
-    Load(String, Box<dyn std::error::Error + Sync + Send>),
+    #[error("{0} `{1}` loader error:\n{2}")]
+    Loader(String, String, Box<dyn std::error::Error + Sync + Send>),
 
     #[error("stage `{0}` case is empty")]
     CaseEmpty(String),
+
+    #[error("step `{0}` create:\n{1}")]
+    Step(String, Box<dyn std::error::Error + Sync + Send>),
 }
 
 pub struct TaskRunner {
-    step_vec: Arc<TailDropVec<(String, Box<dyn Action>)>>,
+    step_vec: Arc<TailDropVec<(String, StepRunner)>>,
     stage_round_no: usize,
     stage_id: Arc<String>,
-    stage_state: TaskState,
+    stage_state: StageState,
 
     pre_ctx: Option<Arc<Map>>,
     #[allow(dead_code)]
     pre_assess: Option<Box<dyn CaseAssess>>,
     #[allow(dead_code)]
-    pre_step_vec: Option<Arc<TailDropVec<(String, Box<dyn Action>)>>>,
+    pre_step_vec: Option<Arc<TailDropVec<(String, StepRunner)>>>,
 
     task_state: TaskState,
 
     def_ctx: Option<Arc<Map>>,
-    assess_report: Box<dyn Report>,
-    case_store: Box<dyn CaseStore>,
+    reporter: Box<dyn TaskReporter>,
+    loader: Box<dyn TaskLoader>,
     id: Arc<TaskIdSimple>,
-    flow_app: Arc<dyn FlowApp>,
+    flow_app: Arc<dyn App>,
     flow: Arc<Flow>,
 }
 
 impl TaskRunner {
     pub fn new(
-        case_store: Box<dyn CaseStore>,
-        assess_report: Box<dyn Report>,
-        flow_app: Arc<dyn FlowApp>,
+        loader: Box<dyn TaskLoader>,
+        reporter: Box<dyn TaskReporter>,
+        flow_app: Arc<dyn App>,
         flow: Arc<Flow>,
         id: Arc<TaskIdSimple>,
     ) -> TaskRunner {
@@ -88,7 +90,7 @@ impl TaskRunner {
             stage_id: Arc::new("".into()),
             stage_round_no: 0,
 
-            stage_state: TaskState::Ok,
+            stage_state: StageState::Ok,
 
             pre_ctx: None,
             pre_assess: None,
@@ -97,8 +99,8 @@ impl TaskRunner {
             task_state: TaskState::Ok,
 
             def_ctx: None,
-            assess_report,
-            case_store,
+            reporter,
+            loader,
             id,
             flow_app,
             flow,
@@ -112,16 +114,20 @@ impl TaskRunner {
     }
 
     pub async fn run(mut self) -> Box<dyn TaskAssess> {
-        trace!("task run start {}", self.id);
+        trace!("task run  {}", self.id);
         let start = Utc::now();
 
-        if let Err(e) = self.assess_report.start(start, self.flow.clone()).await {
-            error!("task run Err {}", self.id);
+        if let Err(e) = self.reporter.start(start).await {
+            error!("task Err  {}", self.id);
             return Box::new(TaskAssessStruct::new(
-                self.id,
+                self.id.clone(),
                 start,
                 Utc::now(),
-                TaskState::Err(Box::new(Report(e))),
+                TaskState::Err(Box::new(Reporter(
+                    "task".to_string(),
+                    self.id.task().to_string(),
+                    e,
+                ))),
             ));
         }
 
@@ -130,9 +136,9 @@ impl TaskRunner {
                "__meta__": self.flow.meta()
             });
             let rc = RenderContext::wraps(rc).unwrap();
-            let rso = render_assign_object(self.flow_app.get_handlebars(), &rc, def_raw, false);
+            let rso = assign_by_render(self.flow_app.get_handlebars(), &rc, def_raw, false);
             if let Err(e) = rso {
-                error!("task run Err {}", self.id);
+                error!("task Err  {}", self.id);
                 return Box::new(TaskAssessStruct::new(
                     self.id.clone(),
                     start,
@@ -144,81 +150,81 @@ impl TaskRunner {
             }
         }
 
-        if let Some(pre_ste_id_vec) = self.flow.pre_step_id_vec() {
-            let pre_step_vec = step_vec_create(
-                self.flow_app.clone(),
-                self.flow.clone(),
-                self.def_ctx.clone(),
-                None,
-                pre_ste_id_vec.into_iter().map(|s| s.to_owned()).collect(),
-                self.id.clone(),
-            )
-            .await;
-            if let Err(e) = pre_step_vec {
-                error!("task run Err {}", self.id);
-                return Box::new(TaskAssessStruct::new(
+        if let Some(pre_step_id_vec) = self.flow.pre_step_id_vec() {
+            if !pre_step_id_vec.is_empty() {
+                let pre_step_vec = step_vec_create(
+                    self.flow_app.clone(),
+                    self.flow.clone(),
+                    pre_step_id_vec.into_iter().map(|s| s.to_owned()).collect(),
                     self.id.clone(),
-                    start,
-                    Utc::now(),
-                    TaskState::Err(Box::new(e)),
-                ));
-            }
+                )
+                .await;
+                if let Err(e) = pre_step_vec {
+                    error!("task Err  {}", self.id);
+                    return Box::new(TaskAssessStruct::new(
+                        self.id.clone(),
+                        start,
+                        Utc::now(),
+                        TaskState::Err(Box::new(e)),
+                    ));
+                }
 
-            let pre_action_vec = Arc::new(TailDropVec::from(pre_step_vec.unwrap()));
-            let pre_arg = pre_arg(
-                self.flow.clone(),
-                self.id.clone(),
-                self.def_ctx.clone(),
-                pre_action_vec.clone(),
-            )
-            .await;
-            if let Err(e) = pre_arg {
-                error!("task run Err {}", self.id);
-                return Box::new(TaskAssessStruct::new(
-                    self.id,
-                    start,
-                    Utc::now(),
-                    TaskState::Err(Box::new(e)),
-                ));
-            }
-
-            let pre_assess = case_run(self.flow_app.as_ref(), pre_arg.unwrap()).await;
-
-            match pre_assess.state() {
-                CaseState::Err(_) => {
-                    error!("task run Err {}", self.id);
+                let pre_step_vec = Arc::new(TailDropVec::from(pre_step_vec.unwrap()));
+                let pre_arg = pre_arg(
+                    self.flow.clone(),
+                    self.id.clone(),
+                    self.def_ctx.clone(),
+                    pre_step_vec.clone(),
+                )
+                .await;
+                if let Err(e) = pre_arg {
+                    error!("task Err  {}", self.id);
                     return Box::new(TaskAssessStruct::new(
                         self.id,
                         start,
                         Utc::now(),
-                        TaskState::Err(Box::new(PreErr)),
+                        TaskState::Err(Box::new(e)),
                     ));
                 }
 
-                CaseState::Fail(v) => {
-                    error!("task run Err {}", self.id);
-                    return Box::new(TaskAssessStruct::new(
-                        self.id,
-                        start,
-                        Utc::now(),
-                        TaskState::Err(Box::new(PreFail(
-                            v.last().unwrap().id().step().to_string(),
-                        ))),
-                    ));
-                }
-                CaseState::Ok(sa_vec) => {
-                    let pre_ctx = pre_ctx_create(sa_vec.as_ref()).await;
-                    self.pre_ctx = Some(Arc::new(pre_ctx));
-                    self.pre_assess = Some(pre_assess);
-                    self.pre_step_vec = Some(pre_action_vec);
+                let pre_assess = case_run(self.flow_app.as_ref(), pre_arg.unwrap()).await;
+
+                match pre_assess.state() {
+                    CaseState::Err(_) => {
+                        error!("task Err  {}", self.id.clone());
+                        return Box::new(TaskAssessStruct::new(
+                            self.id,
+                            start,
+                            Utc::now(),
+                            TaskState::Err(Box::new(PreErr)),
+                        ));
+                    }
+
+                    CaseState::Fail(v) => {
+                        error!("task Err  {}", self.id);
+                        return Box::new(TaskAssessStruct::new(
+                            self.id.clone(),
+                            start,
+                            Utc::now(),
+                            TaskState::Err(Box::new(PreFail(
+                                v.last().unwrap().id().step().to_string(),
+                            ))),
+                        ));
+                    }
+                    CaseState::Ok(sa_vec) => {
+                        let pre_ctx = pre_ctx_create(sa_vec.as_ref()).await;
+                        self.pre_ctx = Some(Arc::new(pre_ctx));
+                        self.pre_assess = Some(pre_assess);
+                        self.pre_step_vec = Some(pre_step_vec);
+                    }
                 }
             }
         };
 
-        let result = self.start_run().await;
+        let result = self.task_run().await;
 
         let task_assess = if let Err(e) = result {
-            error!("task run Err {}", self.id);
+            error!("task Err  {}", self.id);
             TaskAssessStruct::new(
                 self.id.clone(),
                 start,
@@ -228,11 +234,11 @@ impl TaskRunner {
         } else {
             match self.task_state {
                 TaskState::Ok => {
-                    info!("task run Ok {}", self.id.clone());
+                    info!("task Ok   {}", self.id.clone());
                     TaskAssessStruct::new(self.id.clone(), start, Utc::now(), TaskState::Ok)
                 }
                 TaskState::Fail(c) => {
-                    warn!("task run Fail {}", self.id);
+                    warn!("task Fail {}", self.id);
                     TaskAssessStruct::new(
                         self.id.clone(),
                         start,
@@ -241,26 +247,30 @@ impl TaskRunner {
                     )
                 }
                 TaskState::Err(e) => {
-                    error!("task run Err {}", self.id);
+                    error!("task Err  {}", self.id);
                     TaskAssessStruct::new(self.id.clone(), start, Utc::now(), TaskState::Err(e))
                 }
             }
         };
 
-        if let Err(e) = self.assess_report.end(&task_assess).await {
-            error!("task run Err {}", self.id);
+        if let Err(e) = self.reporter.end(&task_assess).await {
+            error!("task Err  {}", self.id);
             return Box::new(TaskAssessStruct::new(
-                self.id,
+                self.id.clone(),
                 start,
                 Utc::now(),
-                TaskState::Err(Box::new(Report(e))),
+                TaskState::Err(Box::new(Reporter(
+                    "task".to_string(),
+                    self.id.task().to_string(),
+                    e,
+                ))),
             ));
         }
 
         Box::new(task_assess)
     }
 
-    async fn start_run(&mut self) -> Result<(), Error> {
+    async fn task_run(&mut self) -> Result<(), Error> {
         let stage_id_vec: Vec<String> = self
             .flow
             .stage_id_vec()
@@ -268,9 +278,9 @@ impl TaskRunner {
             .map(|s| s.to_owned())
             .collect();
         for state_id in stage_id_vec {
-            trace!("task run stage {}, {}", self.id, state_id);
+            trace!("stage run   {}-{}", self.id, state_id);
             self.stage_run(state_id.as_str()).await?;
-            if let TaskState::Fail(_) = self.stage_state {
+            if let StageState::Fail(_) = self.stage_state {
                 if "stage_fail" == self.flow.stage_break_on(state_id.as_str()) {
                     break;
                 }
@@ -281,7 +291,7 @@ impl TaskRunner {
 
     async fn stage_run(&mut self, stage_id: &str) -> Result<(), Error> {
         self.stage_id = Arc::new(stage_id.to_string());
-        self.stage_state = TaskState::Ok;
+        self.stage_state = StageState::Ok;
         let step_id_vec: Vec<String> = self
             .flow
             .stage_step_id_vec(stage_id)
@@ -291,8 +301,6 @@ impl TaskRunner {
         let action_vec = step_vec_create(
             self.flow_app.clone(),
             self.flow.clone(),
-            self.def_ctx.clone(),
-            self.pre_ctx.clone(),
             step_id_vec,
             self.id.clone(),
         )
@@ -300,7 +308,7 @@ impl TaskRunner {
         self.step_vec = Arc::new(TailDropVec::from(action_vec));
 
         let duration = self.flow.stage_duration(stage_id);
-        let srr = self.stage_round_run(stage_id);
+        let srr = self.stage_run_round(stage_id);
         match timeout(duration, srr).await {
             Ok(r) => r?,
             Err(_) => (),
@@ -308,14 +316,13 @@ impl TaskRunner {
         return Ok(());
     }
 
-    async fn stage_round_run(&mut self, stage_id: &str) -> Result<(), Error> {
+    async fn stage_run_round(&mut self, stage_id: &str) -> Result<(), Error> {
         let concurrency = self.flow.stage_concurrency(stage_id);
         let round_max = self.flow.stage_round(stage_id);
         let mut round_count = 0;
         loop {
             self.stage_round_no = round_count;
-            self.stage_data_vec_run_remaining(stage_id, concurrency)
-                .await?;
+            self.stage_run_once(stage_id, concurrency).await?;
             round_count += 1;
             if round_count >= round_max {
                 break;
@@ -324,35 +331,94 @@ impl TaskRunner {
         return Ok(());
     }
 
-    async fn stage_data_vec_run_remaining(
+    async fn stage_run_once(&mut self, stage_id: &str, concurrency: usize) -> Result<(), Error> {
+        let start = Utc::now();
+
+        let mut loader = self
+            .loader
+            .stage(stage_id)
+            .await
+            .map_err(|e| Loader("stage".to_string(), stage_id.to_string(), e))?;
+
+        let mut reporter = self
+            .reporter
+            .stage(stage_id)
+            .await
+            .map_err(|e| Reporter("stage".to_string(), stage_id.to_string(), e))?;
+
+        reporter
+            .start(Utc::now())
+            .await
+            .map_err(|e| Reporter("stage".to_string(), stage_id.to_string(), e))?;
+
+        let result = self
+            .stage_run_io(stage_id, loader.as_mut(), reporter.as_mut(), concurrency)
+            .await;
+
+        let stage_assess = if let Err(e) = result {
+            error!("stage Err  {}-{}", self.id, stage_id);
+            StageAssessStruct::new(
+                stage_id.to_string(),
+                start,
+                Utc::now(),
+                StageState::Err(Box::new(e)),
+            )
+        } else {
+            match &self.stage_state {
+                StageState::Ok => {
+                    info!("stage Ok   {}-{}", self.id.clone(), stage_id);
+                    StageAssessStruct::new(stage_id.to_string(), start, Utc::now(), StageState::Ok)
+                }
+                StageState::Fail(c) => {
+                    warn!("stage Fail {}-{}", self.id.clone(), stage_id);
+                    StageAssessStruct::new(
+                        stage_id.to_string(),
+                        start,
+                        Utc::now(),
+                        StageState::Fail(c.clone()),
+                    )
+                }
+                StageState::Err(_) => unreachable!(),
+            }
+        };
+
+        reporter
+            .end(&stage_assess)
+            .await
+            .map_err(|e| Reporter("stage".to_string(), stage_id.to_string(), e))?;
+
+        match stage_assess.state() {
+            StageState::Ok => Ok(()),
+            StageState::Fail(_) => Ok(()),
+            StageState::Err(_) => unreachable!(),
+        }
+    }
+
+    async fn stage_run_io(
         &mut self,
         stage_id: &str,
+        loader: &mut dyn StageLoader,
+        reporter: &mut dyn StageReporter,
         concurrency: usize,
     ) -> Result<(), Error> {
-        let case_name = self.flow.stage_case_name(stage_id);
-        let mut data_load = self
-            .case_store
-            .create(case_name)
-            .await
-            .map_err(|e| Load(stage_id.to_string(), e))?;
         let mut load_times = 0;
         loop {
-            let case_data_vec: Vec<(String, Value)> = data_load
+            let case_data_vec: Vec<(String, Value)> = loader
                 .load(concurrency)
                 .await
-                .map_err(|e| Load(stage_id.to_string(), e))?;
+                .map_err(|e| Loader("stage".to_string(), stage_id.to_string(), e))?;
             load_times = load_times + 1;
             if case_data_vec.len() == 0 {
                 return if load_times == 1 {
                     Err(CaseEmpty(stage_id.to_string()))
                 } else {
-                    trace!("task run exhaust data {}, {}", self.id, stage_id);
+                    trace!("stage exhaust data {}, {}", self.id, stage_id);
                     Ok(())
                 };
             }
 
             trace!(
-                "task run load data {}, {}, {}",
+                "stage load data {}, {}, {}",
                 self.id,
                 stage_id,
                 case_data_vec.len()
@@ -374,13 +440,13 @@ impl TaskRunner {
                     }
                     CaseState::Ok(_) => String::new(),
                 };
-                self.stage_state = TaskState::Fail(cause.clone());
-                self.task_state = TaskState::Fail(cause);
+                self.stage_state = StageState::Fail(cause.clone());
+                self.task_state = TaskState::Fail(cause.clone());
             }
-            self.assess_report
+            reporter
                 .report(&case_assess_vec)
                 .await
-                .map_err(|e| Report(e))?;
+                .map_err(|e| Reporter("stage".to_string(), stage_id.to_string(), e))?;
         }
     }
 
@@ -391,19 +457,24 @@ impl TaskRunner {
     ) -> Vec<Box<dyn CaseAssess>> {
         let ca_vec = self.case_arg_vec(case_vec);
 
-        let mut case_assess_vec = Vec::<Box<dyn CaseAssess>>::new();
+        let mut case_join_result_vec = Vec::<Result<Box<dyn CaseAssess>, JoinError>>::new();
         let mut futures = vec![];
         for ca in ca_vec {
             let f = case_spawn(self.flow_app.clone(), ca);
             futures.push(f);
             if futures.len() >= concurrency {
                 let case_assess = join_all(futures.split_off(0)).await;
-                case_assess_vec.extend(case_assess);
+                case_join_result_vec.extend(case_assess);
             }
         }
         if !futures.is_empty() {
             let case_assess = join_all(futures).await;
-            case_assess_vec.extend(case_assess);
+            case_join_result_vec.extend(case_assess);
+        }
+
+        let mut case_assess_vec = Vec::with_capacity(case_join_result_vec.len());
+        for res in case_join_result_vec {
+            case_assess_vec.push(res.unwrap());
         }
         case_assess_vec
     }
@@ -433,11 +504,11 @@ async fn pre_arg(
     flow: Arc<Flow>,
     task_id: Arc<TaskIdSimple>,
     def_ctx: Option<Arc<Map>>,
-    pre_action_vec: Arc<TailDropVec<(String, Box<dyn Action>)>>,
+    pre_step_vec: Arc<TailDropVec<(String, StepRunner)>>,
 ) -> Result<CaseArgStruct, Error> {
     Ok(CaseArgStruct::new(
         flow.clone(),
-        pre_action_vec,
+        pre_step_vec,
         Value::Null,
         None,
         def_ctx,
@@ -460,94 +531,49 @@ async fn pre_ctx_create(sa_vec: &Vec<Box<dyn StepAssess>>) -> Map {
 }
 
 async fn step_vec_create(
-    flow_app: Arc<dyn FlowApp>,
+    flow_app: Arc<dyn App>,
     flow: Arc<Flow>,
-    def_ctx: Option<Arc<Map>>,
-    pre_ctx: Option<Arc<Map>>,
     step_id_vec: Vec<String>,
     task_id: Arc<TaskIdSimple>,
-) -> Result<Vec<(String, Box<dyn Action>)>, Error> {
-    let render_context = render_context_create(flow.clone(), def_ctx, pre_ctx);
-    let mut action_vec = vec![];
+) -> Result<Vec<(String, StepRunner)>, Error> {
+    let mut step_vec = vec![];
+    let case_id = Arc::new(CaseIdStruct::new(
+        task_id,
+        Arc::new("create".to_string()),
+        Arc::new("0".to_string()),
+        "0".to_string(),
+    ));
     for sid in step_id_vec {
-        let pr = step_create(
+        let mut arg = ArgStruct::new(
             flow_app.as_ref(),
             flow.as_ref(),
-            &render_context,
-            task_id.clone(),
+            RenderContext::wraps(Value::Object(Map::with_capacity(0)))
+                .map_err(|e| Step(sid.clone(), Box::new(e)))?,
+            case_id.clone(),
             sid.clone(),
-        )
-        .await?;
-        action_vec.push((sid, pr));
+        );
+
+        let pr = step::StepRunner::new(&mut arg)
+            .await
+            .map_err(|e| Error::Step(sid.clone(), Box::new(e)))?;
+        step_vec.push((sid, pr));
     }
-    Ok(action_vec)
+    Ok(step_vec)
 }
 
-fn render_context_create(
-    flow: Arc<Flow>,
-    def_ctx: Option<Arc<Map>>,
-    _pre_ctx: Option<Arc<Map>>,
-) -> RenderContext {
-    let mut render_data: Map = Map::new();
-
-    render_data.insert("__meta__".to_owned(), Value::Object(flow.meta().clone()));
-    if let Some(def_ctx) = def_ctx {
-        render_data.insert("def".to_owned(), Value::Object(def_ctx.as_ref().clone()));
-    }
-
-    return RenderContext::wraps(render_data).unwrap();
-}
-
-async fn step_create(
-    flow_app: &dyn FlowApp,
-    flow: &Flow,
-    render_ctx: &RenderContext,
-    task_id: Arc<TaskIdSimple>,
-    step_id: String,
-) -> Result<Box<dyn Action>, Error> {
-    let let_raw = flow.step_let(step_id.as_ref());
-    let let_value = match let_raw {
-        Some(let_raw) => {
-            let let_value =
-                render_assign_object(flow_app.get_handlebars(), render_ctx, let_raw, true)
-                    .map_err(|e| Render(format!("step.{}.let", step_id), e))?;
-            Some(let_value)
-        }
-        None => None,
-    };
-
-    let action = flow.step_exec_action(step_id.as_ref());
-    let create_arg = CreateArgStruct::new(
-        flow,
-        flow_app.get_handlebars(),
-        let_value,
-        task_id,
-        action.into(),
-        step_id.clone(),
-    );
-
-    flow_app
-        .get_action_factory()
-        .create(&create_arg)
-        .await
-        .map_err(|e| StepCreate(step_id, e))
-}
-
-async fn case_run(flow_ctx: &dyn FlowApp, case_arg: CaseArgStruct) -> Box<dyn CaseAssess> {
+async fn case_run(flow_ctx: &dyn App, case_arg: CaseArgStruct) -> Box<dyn CaseAssess> {
     Box::new(case::run(flow_ctx, case_arg).await)
 }
 
-fn case_spawn(
-    flow_ctx: Arc<dyn FlowApp>,
-    case_arg: CaseArgStruct,
-) -> JoinHandle<Box<dyn CaseAssess>> {
-    let builder = Builder::new()
-        .name(format!("case_{}", case_arg.id()))
-        .spawn(case_run_arc(flow_ctx, case_arg));
-    return builder.unwrap();
+fn case_spawn(flow_ctx: Arc<dyn App>, case_arg: CaseArgStruct) -> JoinHandle<Box<dyn CaseAssess>> {
+    spawn(case_run_arc(flow_ctx, case_arg))
 }
 
-async fn case_run_arc(flow_ctx: Arc<dyn FlowApp>, case_arg: CaseArgStruct) -> Box<dyn CaseAssess> {
-    CTX_ID.with(|cid| cid.replace(case_arg.id().to_string()));
-    case_run(flow_ctx.as_ref(), case_arg).await
+async fn case_run_arc(flow_ctx: Arc<dyn App>, case_arg: CaseArgStruct) -> Box<dyn CaseAssess> {
+    CTX_ID
+        .scope(
+            case_arg.id().to_string(),
+            case_run(flow_ctx.as_ref(), case_arg),
+        )
+        .await
 }
